@@ -5,7 +5,7 @@
     Author:        Jan Wielemaker and Peter Ludemann
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2022-2023, SWI-Prolog Solutions b.v.
+    Copyright (c)  2022-2024, SWI-Prolog Solutions b.v.
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -61,7 +61,7 @@ user:portray(MyBlob) :-
 %     run_tests([ cpp,
 %                 cpp_atommap,
 %                 cpp_map_str_str
-% 	      ]).
+%	      ]).
 
 test_cpp :-
     run_tests.
@@ -190,6 +190,14 @@ test(can_unify) :-
 test(can_unify, fail) :-
     can_unify(a(1), a(2)).
 
+test(can_unify_ffi, [true(X\==Y)]) :-
+    can_unify_ffi(f(X), f(Y)).
+test(can_unify_ffi) :-
+    can_unify_ffi(a(X), a(1)),
+    assertion(var(X)).
+test(can_unify_ffi, fail) :-
+    can_unify_ffi(a(1), a(2)).
+
 test(call_chars, Out=="1") :-
     with_output_to(string(Out), call_chars("X=1, write(X)")).
 test(call_chars, Out=="1") :-
@@ -624,6 +632,8 @@ test(name_to_terms, fail) :-
     name_to_terms("foo", _, _).
 test(name_to_terms, fail) :-
     name_to_terms("two", _, "deux").
+test(name_to_terms, error(type_error('atom or string',A))) :-
+     name_to_terms(A, 1, 2).
 
 test(name_to_terms2, [T1,T2] = [1,"eins"]) :-
     name_to_terms2("one", T1, T2).
@@ -886,6 +896,87 @@ test(blob_portray, S == "MyBlob(closed)") :-
     close_my_blob(B),
     with_output_to(string(S), print(B)).
 
+test(nchars_flags, F-S == 0x43f-"xinteger,all") :-
+    nchars_flags([xinteger,all,atomic,number], F),
+    nchars_flags_string(F, S).
+test(nchars_flags, F-S == 0x37-"all") :-
+    nchars_flags([all,atomic,number], F),
+    nchars_flags_string(F, S).
+test(nchars_flags, F-S == 0x7ff-"xinteger,all,variable,write,write_canonical,writeq") :-
+    nchars_flags([atom,string,integer,list,rational,float,variable,number,atomic,write,write_canonical,writeq,all,xinteger], F),
+    nchars_flags_string(F, S).
+test(nchars_flags, F-S == 0x37-"all") :-
+    nchars_flags([atomic,list], F),
+    nchars_flags_string(F, S).
+test(nchars_flags, F-S == 0x33-"atomic") :-
+    nchars_flags([number,atom,string], F),
+    nchars_flags_string(F, S).
+
+test(nchars, S-FS-FS2 == "123"-"atomic"-"atomic") :-
+    nchars_flags([atomic], F),
+    nchars_flags_string(F, FS),
+    get_nchars_string(123, F, S, FS2).
+test(nchars, S-F == "123"-"atomic") :-
+    get_nchars_string(123, [atomic], S, F).
+test(nchars, error(type_error(atomic,f(a)))) :-
+    get_nchars_string(f(a), [atomic], _, _).
+test(nchars, S-F == "+(a,b)"-"all,write_canonical") :-
+    get_nchars_string(a+b, [write_canonical,all], S, F).
+
+% The flags to PlTerm::as_string are [all,writeq].
+% TODO: try more flag combinations
+
+test(nchars, [S-F == "a b"-"all,writeq"]) :-
+    get_nchars_string('a b', [all,writeq], S, F).
+test(nchars, [S-F == "a b"-"all,writeq"]) :-
+    get_nchars_string("a b", [all,writeq], S, F).
+test(nchars, [S-F == "'a b'"-"writeq"]) :-
+    get_nchars_string('a b', [writeq], S, F).
+test(nchars, [S-F == "\"a b\""-"writeq"]) :-
+    get_nchars_string("a b", [writeq], S, F).
+test(nchars, S-F == "f('a b')"-"all,writeq") :-
+    get_nchars_string(f('a b'), [all,writeq], S, F).
+test(nchars, S-F == "f(\"a b\")"-"all,writeq") :-
+    get_nchars_string(f("a b"), [all,writeq], S, F).
+test(nchars, S-F == "f(A,_,b,A)"-"writeq") :-
+    get_nchars_string(f(X,_,b,X), [writeq], S, F).
+
+test(nchars, S-F == "an atom"-"atom") :-
+    get_nchars_string('an atom', [atom], S, F).
+test(nchars, S-F == "a string"-"string") :-
+    get_nchars_string("a string", [string], S, F).
+test(nchars, S-F == "abc"-"list") :-
+    get_nchars_string([a,b,c], [list], S, F).
+test(nchars, S-F == "abcd"-"list") :-
+    get_nchars_string([0'a,0'b,0'c,0'd], [list], S, F).
+test(nchars, S-F == "10r3"-"rational") :-
+    get_nchars_string(20r6, [rational], S, F).
+test(nchars, S-F == "ar3"-"xinteger,rational") :-
+    get_nchars_string(20r6, [xinteger,rational], S, F).
+test(nchars, S-F == "14"-"xinteger,rational") :-
+    get_nchars_string(20, [xinteger,rational], S, F).
+test(nchars, S-F == "14"-"xinteger") :-
+    get_nchars_string(20, [xinteger], S, F).
+test(nchars, S-F == "0.25"-"xinteger,float") :-
+    get_nchars_string(0.25, [xinteger,float], S, F).
+
+% TODO: the type_error always shows atom - but making it more accurate
+%       would be more work than it's worth.
+test(nchars, error(type_error(atom,0.25))) :-
+    get_nchars_string(0.25, [xinteger], _S, _F).
+
+test(lookup_unify, N == 1) :-
+    lookup_unify(item(one, N)).
+test(lookup_unify, S == three) :-
+    lookup_unify(item(S, 3)).
+test(lookup_unify, fail) :-
+    lookup_unify(xxx).
+
+test(#, S == "abc") :-
+    #(abc, S).
+test(#, S == "foo(abc)") :-
+    #(foo(abc), S).
+
 compare_write_form(Compare, A, B) :-
     with_output_to(string(Astr), write(A)),
     with_output_to(string(Bstr), write(B)),
