@@ -2,8 +2,9 @@
 
     Author:        Matt Lilley and Markus Triska
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2004-2017, SWI-Prolog Foundation
+    Copyright (c)  2004-2024, SWI-Prolog Foundation
                               VU University Amsterdam
+			      SWI-Prolog Solutions b.v.
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -32,6 +33,7 @@
     POSSIBILITY OF SUCH DAMAGE.
 */
 
+#define _CRT_SECURE_NO_WARNINGS 1
 #include <config.h>
 #include <assert.h>
 #include <string.h>
@@ -48,6 +50,8 @@
 #ifdef HAVE_OPENSSL_KDF_H
 #include <openssl/kdf.h>
 #endif
+#include "crypt_blowfish.h"
+
 #include "cryptolib.c"
 
 static atom_t ATOM_sslv23;
@@ -115,7 +119,7 @@ pl_crypto_n_random_bytes(term_t tn, term_t tout)
   if ( !(buffer = malloc(len)) )
     return PL_resource_error("memory");
 
-  if ( RAND_bytes(buffer, len) == 0 )
+  if ( RAND_bytes(buffer, (int)len) == 0 )
   { free(buffer);
     return raise_ssl_error(ERR_get_error());
   }
@@ -149,18 +153,20 @@ typedef struct hash_context
 
   EVP_MD_CTX     *ctx;
 #if defined USE_EVP_API
-  EVP_MAC         *mac;
-  EVP_MAC_CTX     *mac_ctx;
+  EVP_MAC	 *mac;
+  EVP_MAC_CTX    *mac_ctx;
 #else
   HMAC_CTX       *mac_ctx;
 #endif
   char           *mac_key;
+  size_t	  mac_key_len;
 } PL_CRYPTO_HASH_CONTEXT;
 
 static void
 free_crypto_hash_context(PL_CRYPTO_HASH_CONTEXT *c)
 { EVP_MD_CTX_free(c->ctx);
-  free(c->mac_key);
+  if ( c->mac_key )
+    PL_free(c->mac_key);	      /* allocated using BUF_MALLOC */
 #ifdef USE_EVP_API
   EVP_MAC_free(c->mac);
   EVP_MAC_CTX_free(c->mac_ctx);
@@ -333,9 +339,11 @@ hash_options(term_t options, PL_CRYPTO_HASH_CONTEXT *result)
         char *key;
 
         if ( !PL_get_nchars(a, &key_len, &key,
-                      CVT_ATOM|CVT_STRING|CVT_LIST|CVT_EXCEPTION) )
+			    CVT_ATOM|CVT_STRING|CVT_LIST|
+			    CVT_EXCEPTION|BUF_MALLOC) )
           return FALSE;
-        result->mac_key = ssl_strdup(key);
+        result->mac_key = key;
+	result->mac_key_len = key_len;
       } else if ( aname == ATOM_close_parent )
       { if ( !PL_get_bool_ex(a, &result->close_parent) )
           return FALSE;
@@ -360,21 +368,12 @@ hash_options(term_t options, PL_CRYPTO_HASH_CONTEXT *result)
 
 static foreign_t
 pl_crypto_hash_context_new(term_t tcontext, term_t options)
-{
-  PL_CRYPTO_HASH_CONTEXT *context = NULL;
+{ PL_CRYPTO_HASH_CONTEXT *context;
 
-  context = malloc(sizeof(*context));
-
-  if ( !context )
-    return FALSE;
-
-  context->magic    = HASH_CONTEXT_MAGIC;
-  context->ctx      = NULL;
-  context->mac_ctx  = NULL;
-  context->mac_key  = NULL;
-
-  context->parent_stream = NULL;
-  context->hash_stream   = NULL;
+  if ( !(context = malloc(sizeof(*context))) )
+    return PL_resource_error("memory");
+  memset(context, 0, sizeof(*context));
+  context->magic = HASH_CONTEXT_MAGIC;
 
   if ( !hash_options(options, context) )
     return FALSE;
@@ -394,7 +393,7 @@ pl_crypto_hash_context_new(term_t tcontext, term_t options)
     params[0] = OSSL_PARAM_construct_utf8_string("digest", (char *)EVP_MD_name(context->algorithm), 0);
     params[1] = OSSL_PARAM_construct_end();
     if ( !EVP_MAC_init(context->mac_ctx,
-                       (unsigned char*)context->mac_key, strlen(context->mac_key),
+                       (unsigned char*)context->mac_key, context->mac_key_len,
                        params) )
     { EVP_MAC_CTX_free(context->mac_ctx);
       EVP_MAC_free(context->mac);
@@ -405,7 +404,7 @@ pl_crypto_hash_context_new(term_t tcontext, term_t options)
   if ( context->mac_key )
   { context->mac_ctx = HMAC_CTX_new();
     if ( !HMAC_Init_ex(context->mac_ctx,
-                       context->mac_key, strlen(context->mac_key),
+                       context->mac_key, context->mac_key_len,
                        context->algorithm, NULL) )
     { HMAC_CTX_free(context->mac_ctx);
       return FALSE;
@@ -433,17 +432,20 @@ pl_crypto_hash_context_copy(term_t tin, term_t tout)
   if ( !get_hash_context(tin, &in) )
     return FALSE;
 
-  out = malloc(sizeof(*out));
-
-  if ( !out )
-    return FALSE;
-
+  if ( !(out = malloc(sizeof(*out))) )
+    return PL_resource_error("memory");
+  memset(out, 0, sizeof(*out));
   out->magic = HASH_CONTEXT_MAGIC;
-  out->mac_key = ssl_strdup(in->mac_key);
-
   out->encoding = in->encoding;
   out->algorithm = in->algorithm;
 
+  if ( in->mac_key )
+  { char *tmp = PL_malloc(in->mac_key_len+1);
+    memcpy(tmp, in->mac_key, in->mac_key_len+1);
+    out->mac_key = tmp;
+    out->mac_key_len = in->mac_key_len;
+  }
+
   out->ctx = in->ctx ? EVP_MD_CTX_new() : NULL;
   if ( out->ctx )
   { if ( !EVP_DigestInit_ex(out->ctx, out->algorithm, NULL) )
@@ -464,7 +466,7 @@ pl_crypto_hash_context_copy(term_t tin, term_t tout)
 
   if ( out->mac_ctx )
   { if ( !HMAC_Init_ex(out->mac_ctx,
-                       out->mac_key, strlen(out->mac_key),
+                       out->mac_key, out->mac_key_len,
                        out->algorithm, NULL) )
     { HMAC_CTX_free(out->mac_ctx);
       return FALSE;
@@ -686,13 +688,14 @@ pl_crypto_stream_hash_context(term_t stream, term_t tcontext)
                  *    Hashes of passwords   *
                  ****************************/
 
+#define PBKDF2_DIGEST_LEN 64
+
 static foreign_t
-pl_crypto_password_hash(term_t tpw, term_t tsalt, term_t titer, term_t tdigest)
+pl_crypto_password_hash_pbkdf2(term_t tpw, term_t tsalt, term_t titer, term_t tdigest)
 { char *pw, *salt;
   size_t pwlen, saltlen;
   int iter;
-  const int DIGEST_LEN = 64;
-  unsigned char digest[DIGEST_LEN];
+  unsigned char digest[PBKDF2_DIGEST_LEN];
 
   if ( !PL_get_nchars(tpw, &pwlen, &pw,
                       CVT_ATOM|CVT_STRING|CVT_LIST|CVT_EXCEPTION|REP_UTF8) ||
@@ -700,11 +703,32 @@ pl_crypto_password_hash(term_t tpw, term_t tsalt, term_t titer, term_t tdigest)
        !PL_get_integer_ex(titer, &iter) )
     return FALSE;
 
-  PKCS5_PBKDF2_HMAC((const char *) pw, pwlen,
-                    (const unsigned char *) salt, saltlen,
-                    iter, EVP_sha512(), DIGEST_LEN, digest);
+  PKCS5_PBKDF2_HMAC((const char *) pw, (int)pwlen,
+                    (const unsigned char *) salt, (int)saltlen,
+                    iter, EVP_sha512(), PBKDF2_DIGEST_LEN, digest);
 
-  return PL_unify_list_ncodes(tdigest, DIGEST_LEN, (char *) digest);
+  return PL_unify_list_ncodes(tdigest, PBKDF2_DIGEST_LEN, (char *) digest);
+}
+
+#define BCRYPT_DIGEST_LEN (7 + 22 + 31 + 1)
+
+static foreign_t
+pl_crypto_password_hash_bcrypt(term_t tpw, term_t tsetting, term_t tdigest)
+{ char *pw, *setting;
+  size_t pwlen, settinglen;
+  char digest[BCRYPT_DIGEST_LEN];
+
+  if ( !PL_get_nchars(tpw, &pwlen, &pw,
+                      CVT_ATOM|CVT_STRING|CVT_LIST|CVT_EXCEPTION|REP_UTF8) ||
+       !PL_get_nchars(tsetting, &settinglen, &setting,
+                      CVT_ATOM|CVT_STRING|CVT_LIST|CVT_EXCEPTION|REP_UTF8) )
+    return FALSE;
+
+  char* ret = _crypt_blowfish_rn(pw, setting, (char *) digest, BCRYPT_DIGEST_LEN);
+  if ( ret == NULL )
+    return PL_domain_error("setting", tsetting);
+
+  return PL_unify_chars(tdigest, PL_ATOM | REP_UTF8, BCRYPT_DIGEST_LEN - 1, (char *) digest);
 }
 
 static foreign_t
@@ -744,9 +768,9 @@ pl_crypto_data_hkdf(term_t tkey, term_t tsalt, term_t tinfo, term_t talg,
 
   if ( (EVP_PKEY_derive_init(pctx) > 0) &&
        (EVP_PKEY_CTX_set_hkdf_md(pctx, alg) > 0) &&
-       (EVP_PKEY_CTX_set1_hkdf_salt(pctx, (unsigned char*)salt, saltlen) > 0) &&
-       (EVP_PKEY_CTX_set1_hkdf_key(pctx, (unsigned char*)key, keylen) > 0) &&
-       (EVP_PKEY_CTX_add1_hkdf_info(pctx, (unsigned char*)info, infolen) > 0) &&
+       (EVP_PKEY_CTX_set1_hkdf_salt(pctx, (unsigned char*)salt, (int)saltlen) > 0) &&
+       (EVP_PKEY_CTX_set1_hkdf_key(pctx, (unsigned char*)key, (int)keylen) > 0) &&
+       (EVP_PKEY_CTX_add1_hkdf_info(pctx, (unsigned char*)info, (int)infolen) > 0) &&
        (EVP_PKEY_derive(pctx, out, &outlen) > 0) )
   { int rc = PL_unify_list_ncodes(tout, outlen, (char *) out);
     free(out);
@@ -842,7 +866,7 @@ recover_ec(term_t t, ECKEY **rec)
 #ifdef USE_EVP_API
         EVP_PKEY_set_octet_string_param(key, "pub", (const unsigned char*) codes, codes_len)
 #else
-       (key = o2i_ECPublicKey(&key, (const unsigned char**) &codes, codes_len)) 
+       (key = o2i_ECPublicKey(&key, (const unsigned char**) &codes, codes_len))
 #endif
   )
   { *rec = key;
@@ -869,14 +893,14 @@ recover_rsa(term_t t, RSAKEY** keyp)
 #endif
 
 #if OPENSSL_VERSION_NUMBER < 0x10100000L || defined(LIBRESSL_VERSION_NUMBER)
-  if ( get_bn_arg(1, t, &rsa->n) &&
-       get_bn_arg(2, t, &rsa->e) &&
-       get_bn_arg(3, t, &rsa->d) &&
-       get_bn_arg(4, t, &rsa->p) &&
-       get_bn_arg(5, t, &rsa->q) &&
-       get_bn_arg(6, t, &rsa->dmp1) &&
-       get_bn_arg(7, t, &rsa->dmq1) &&
-       get_bn_arg(8, t, &rsa->iqmp)
+  if ( get_bn_arg(1, t, &key->n) &&
+       get_bn_arg(2, t, &key->e) &&
+       get_bn_arg(3, t, &key->d) &&
+       get_bn_arg(4, t, &key->p) &&
+       get_bn_arg(5, t, &key->q) &&
+       get_bn_arg(6, t, &key->dmp1) &&
+       get_bn_arg(7, t, &key->dmq1) &&
+       get_bn_arg(8, t, &key->iqmp)
      )
   {
 #else
@@ -1097,7 +1121,7 @@ pl_ecdsa_sign(term_t Private, term_t Data, term_t Enc, term_t Signature)
 		                 signature, &signature_len,
 		                 data, (unsigned int)data_len);
   EVP_PKEY_CTX_free(sign_ctx);
-  if (!rc) 
+  if (!rc)
     return raise_ssl_error(ERR_get_error());
 #else
   sig = ECDSA_do_sign(data, (unsigned int)data_len, key);
@@ -1135,7 +1159,7 @@ pl_ecdsa_verify(term_t Public, term_t Data, term_t Enc, term_t Signature)
 
   copy = signature;
 
-  if ( !(sig = d2i_ECDSA_SIG(NULL, &copy, signature_len)) )
+  if ( !(sig = d2i_ECDSA_SIG(NULL, &copy, (int)signature_len)) )
     return FALSE;
 
 #ifdef USE_EVP_API
@@ -1662,7 +1686,7 @@ pl_crypto_data_decrypt(term_t ciphertext_t, term_t algorithm_t,
 #ifdef EVP_CTRL_AEAD_SET_TAG
   if ( PL_get_nchars(authtag_t, &authlen, &authtag, CVT_LIST) &&
        ( authlen > 0 ) )
-  { if ( !EVP_CIPHER_CTX_ctrl(ctx, EVP_CTRL_AEAD_SET_TAG, authlen, authtag) )
+  { if ( !EVP_CIPHER_CTX_ctrl(ctx, EVP_CTRL_AEAD_SET_TAG, (int)authlen, authtag) )
     { EVP_CIPHER_CTX_free(ctx);
       return raise_ssl_error(ERR_get_error());
     }
@@ -1672,7 +1696,7 @@ pl_crypto_data_decrypt(term_t ciphertext_t, term_t algorithm_t,
   EVP_CIPHER_CTX_set_padding(ctx, padding);
   plaintext = PL_malloc(cipher_length + EVP_CIPHER_block_size(cipher));
   if ( EVP_DecryptUpdate(ctx, (unsigned char*)plaintext, &plain_length,
-			 (unsigned char*)ciphertext, cipher_length) == 1 )
+			 (unsigned char*)ciphertext, (int)cipher_length) == 1 )
   { int last_chunk = plain_length;
     int rc;
     rc = EVP_DecryptFinal_ex(ctx, (unsigned char*)(plaintext + plain_length),
@@ -1696,6 +1720,8 @@ pl_crypto_data_decrypt(term_t ciphertext_t, term_t algorithm_t,
   return raise_ssl_error(ERR_get_error());
 }
 
+#define MAX_AUTHLEN 256
+
 static foreign_t
 pl_crypto_data_encrypt(term_t plaintext_t, term_t algorithm_t,
                        term_t key_t, term_t iv_t,
@@ -1715,7 +1741,6 @@ pl_crypto_data_encrypt(term_t plaintext_t, term_t algorithm_t,
   int rep = REP_UTF8;
   int padding = 1;
   int authlen;
-  const int MAX_AUTHLEN = 256;
 #ifdef EVP_CTRL_AEAD_SET_TAG
   char authtag[MAX_AUTHLEN];
 #endif
@@ -1742,7 +1767,7 @@ pl_crypto_data_encrypt(term_t plaintext_t, term_t algorithm_t,
   EVP_CIPHER_CTX_set_padding(ctx, padding);
   ciphertext = PL_malloc(plain_length + EVP_CIPHER_block_size(cipher));
   if ( EVP_EncryptUpdate(ctx, (unsigned char*)ciphertext, &cipher_length,
-                         (unsigned char*)plaintext, plain_length) == 1 )
+                         (unsigned char*)plaintext, (int)plain_length) == 1 )
   { int last_chunk;
     int rc;
 
@@ -2360,7 +2385,8 @@ install_crypto4pl(void)
   PL_register_foreign("_crypto_stream_hash_context", 2,
                       pl_crypto_stream_hash_context, 0);
 
-  PL_register_foreign("_crypto_password_hash", 4, pl_crypto_password_hash, 0);
+  PL_register_foreign("_crypto_password_hash_pbkdf2", 4, pl_crypto_password_hash_pbkdf2, 0);
+  PL_register_foreign("_crypto_password_hash_bcrypt", 3, pl_crypto_password_hash_bcrypt, 0);
   PL_register_foreign("_crypto_data_hkdf", 7, pl_crypto_data_hkdf, 0);
 
   PL_register_foreign("_crypto_ecdsa_sign", 4, pl_ecdsa_sign, 0);
