@@ -17,6 +17,9 @@ def isstr(s):
     else:
         return False
 
+def dict0():
+    return {}
+
 def dict1():
     return {"a":1, 2:2}
 
@@ -103,8 +106,8 @@ def squares(start, stop):
 
 # Test for WFS
 
-def shaves(truth=PLAIN_TRUTHVALS):
-    return [*query("russel:shaves(X,Y)", truth=truth)]
+def shaves(truth_vals=PLAIN_TRUTHVALS):
+    return [*query("russel:shaves(X,Y)", truth_vals=truth_vals)]
 
 # Benchmarking support
 
